/*****************************************************************************

  Program:      rssFeed.i
  
  Written By:   Gordon Campbell
  Written On:   June 2005 - March 2006
  
  Description:  Defines the RSS Feed functionality
  
*****************************************************************************/

/* ----------------------------- Definitions ------------------------------- */
&GLOBAL-DEFINE ChannelOptions language,copyright,managingEditor,webMaster~
,PubDate,lastBuildDate,category,generator,docs,cloud,ttl,image,rating,~
textInput,skipHours,skipDays

&GLOBAL-DEFINE ItemOptions link,author,category,comments,,enclosure,guid,~
pubDate,source

DEFINE TEMP-TABLE ttRSSfeed NO-UNDO
  FIELD FeedID              AS INTEGER
  FIELD FeedFilename        AS CHARACTER
INDEX FeedID  AS PRIMARY UNIQUE
      FeedID
INDEX FeedFileName
      FeedFileName.

DEFINE TEMP-TABLE ttRSSchannel NO-UNDO
  FIELD FeedId              AS INTEGER
  FIELD ChannelID           AS INTEGER
  FIELD ChannelTitle        AS CHARACTER
  FIELD ChannelLink         AS CHARACTER
  FIELD ChannelDescription  AS CHARACTER
INDEX FeedChannel AS PRIMARY UNIQUE
      FeedId
      ChannelID
INDEX ChannelTitle
      ChannelTitle.

DEFINE TEMP-TABLE ttRSSitem NO-UNDO
  FIELD FeedId              AS INTEGER
  FIELD ChannelID           AS INTEGER
  FIELD ItemID              AS INTEGER
  FIELD ItemTitle           AS CHARACTER
  FIELD ItemDesc            AS CHARACTER
INDEX FeedChannelItem AS PRIMARY UNIQUE
      FeedId
      ChannelID
      ItemID.

DEFINE TEMP-TABLE ttRSSoption NO-UNDO
  FIELD FeedId              AS INTEGER
  FIELD ChannelID           AS INTEGER
  FIELD ItemID              AS INTEGER
  FIELD OptionName          AS CHARACTER
  FIELD OptionValue         AS CHARACTER
INDEX FeedChannelItemOption AS PRIMARY UNIQUE
      FeedId
      ChannelID
      ItemID
      OptionName.

DEFINE STREAM sRSS.

/* ------------------------- Function Declarations ------------------------- */

FUNCTION CreateRSSFeed RETURN INTEGER
   (INPUT pFeedFileName AS CHARACTER):

  DEFINE VARIABLE iFeed AS INTEGER NO-UNDO.

  DEFINE BUFFER BttRSSfeed FOR ttRSSfeed.

  /* If the Feed Name has already been used then return -1 to indicate that
     it has already been used */
  FIND FIRST BttRSSfeed WHERE BttRSSfeed.FeedFileName = pFeedFileName NO-LOCK NO-ERROR.  
  IF AVAIL BttRSSfeed THEN RETURN -1.

  FIND LAST BttRSSfeed NO-LOCK NO-ERROR.
  iFeed = IF NOT AVAIL BttRSSfeed THEN 1 ELSE BttRSSfeed.FeedId + 1.

  CREATE ttRSSfeed.
  ASSIGN ttRSSfeed.FeedId   = iFeed
         ttRSSfeed.FeedFileName = pFeedFileName.

  RETURN iFeed.
END FUNCTION. /* CreateFeed */

FUNCTION SaveRSSFeed RETURN INTEGER
   (INPUT pFeedID AS INTEGER):

  /* If the FeedID wasn't found then return -2 to indicate that it wasn't 
     found */
  FIND FIRST ttRSSfeed WHERE ttRSSfeed.FeedID = pFeedId NO-ERROR.  
  IF NOT AVAIL ttRSSfeed THEN RETURN -2.

  OUTPUT STREAM sRSS TO VALUE(ttRSSfeed.FeedFileName).
  
  /* RSS Start */
  PUT STREAM sRSS UNFORMATTED
      "<?xml version=~"1.0~" ?>" SKIP
      "<rss version=~"2.0~">" SKIP.

  FOR EACH ttRSSchannel 
      WHERE ttRSSchannel.FeedID = pFeedID:
    
    /* Channel Start */
    PUT STREAM sRSS UNFORMATTED
        "<channel>" SKIP
        "<title>" ttRSSchannel.ChannelTitle "</title>" SKIP
        "<link>" ttRSSchannel.ChannelLink "</link>" SKIP
        "<description>" ttRSSchannel.ChannelLink "</description>" SKIP.

    /* Output Channel Optional elements */
    FOR EACH ttRSSoption
       WHERE ttRSSoption.FeedID    = pFeedID
         AND ttRSSoption.ChannelID = ttRSSChannel.ChannelID
         AND ttRSSoption.ItemID    = -1:
      PUT STREAM sRSS UNFORMATTED
          "<"   ttRSSoption.OptionName ">"
                ttRSSoption.OptionValue
          "<~/" ttRSSoption.OptionName ">" SKIP.

    END.

    FOR EACH ttRSSitem
        WHERE ttRSSitem.FeedID    = pFeedID
          AND ttRSSitem.ChannelID = ttRSSchannel.ChannelID:

      /* Item Start */
      PUT STREAM sRSS UNFORMATTED
          "<item>" SKIP.

      IF ttRSSitem.ItemTitle <> "" THEN
        PUT STREAM sRSS UNFORMATTED
            "<title>" ttRSSitem.ItemTitle "</title>" SKIP.

      IF ttRSSitem.ItemDesc <> "" THEN
        PUT STREAM sRSS UNFORMATTED
            "<description>" ttRSSitem.ItemDesc "</description>" SKIP.

      /* Output Item Optional elements */
      FOR EACH ttRSSoption
         WHERE ttRSSoption.FeedID    = pFeedID
           AND ttRSSoption.ChannelID = ttRSSchannel.ChannelID
           AND ttRSSoption.ItemID    = ttRSSitem.ItemID:
        PUT STREAM sRSS UNFORMATTED
            "<"   ttRSSoption.OptionName ">"
                  ttRSSoption.OptionValue
            "<~/" ttRSSoption.OptionName ">" SKIP.

      END.

      /* Item End */
      PUT STREAM sRSS UNFORMATTED
          "</item>" SKIP.

    END. /* each ttRSSitem */

    /* Channel End */
    PUT STREAM sRSS UNFORMATTED
        "</channel>" SKIP.

  END. /* each ttRSSchannel */

  /* RSS End */
  PUT STREAM sRSS UNFORMATTED
      "</rss>" SKIP.

  OUTPUT STREAM sRSS CLOSE.

  RETURN 1.  /* To indicate that it was written okay */
END FUNCTION. /* SaveRSSFeed */

FUNCTION ClearRSSFeed RETURN INTEGER
   (INPUT pFeedID AS INTEGER):

  DEFINE VARIABLE iFeed AS INTEGER NO-UNDO.

  /* If the FeedID wasn't found then return -2 to indicate that it wasn't 
     found */
  FIND FIRST ttRSSfeed WHERE ttRSSfeed.FeedID = pFeedId NO-ERROR.  
  IF NOT AVAIL ttRSSfeed THEN RETURN -2.

  FOR EACH ttRSSchannel WHERE ttRSSchannel.FeedID = pFeedID:
    DELETE ttRSSchannel.
  END.

  FOR EACH ttRSSitem WHERE ttRSSitem.FeedID = pFeedID:
    DELETE ttRSSitem.
  END.

  DELETE ttRSSfeed.

  RETURN 0.  /* Indicating that it was deleted */
END FUNCTION. /* ClearFeed */

FUNCTION CreateRSSChannel RETURN INTEGER
   (INPUT pFeedID       AS INTEGER,
    INPUT pChannelTitle AS CHARACTER,
    INPUT pChannelLink  AS CHARACTER,
    INPUT pChannelDesc  AS CHARACTER):

  DEFINE VARIABLE iChannel AS INTEGER NO-UNDO.

  DEFINE BUFFER BttRSSchannel FOR ttRSSchannel.

  /* If the FeedID wasn't found then return -1 to indicate that it wasn't 
     found */
  FIND FIRST ttRSSfeed WHERE ttRSSfeed.FeedID = pFeedId NO-ERROR.  
  IF NOT AVAIL ttRSSfeed THEN RETURN -2.

  IF pChannelTitle = "" THEN
    RETURN -5.

  IF pChannelLink = "" THEN
    RETURN -6.

  IF pChannelDesc = "" THEN
    RETURN -7.

  /* If the Chanel Name has already been used then return -1 to indicate that
     it has already been used */
  FIND FIRST BttRSSchannel
       WHERE BttRSSchannel.FeedID      = pFeedID 
         AND BttRSSchannel.ChannelTitle = pChannelTitle NO-LOCK NO-ERROR.  
  IF AVAIL BttRSSchannel THEN RETURN -3.

  FIND LAST BttRSSchannel NO-LOCK NO-ERROR.
  iChannel = IF NOT AVAIL BttRSSchannel THEN 1 ELSE BttRSSchannel.ChannelId + 1.

  CREATE ttRSSchannel.
  ASSIGN ttRSSchannel.FeedID       = pFeedID
         ttRSSchannel.ChannelId    = iChannel
         ttRSSchannel.ChannelTitle = pChannelTitle
         ttRSSchannel.ChannelLink  = pChannelLink
         ttRSSchannel.ChannelDesc  = pChannelDesc.

  RETURN iChannel.
END FUNCTION. /* CreateChannel */

FUNCTION SetRSSChannelOption RETURN INTEGER
   (INPUT pFeedID       AS INTEGER,
    INPUT pChannelID    AS INTEGER,
    INPUT pOptionName   AS CHARACTER,
    INPUT pOptionValue  AS CHARACTER):

  /* If the FeedID wasn't found then return -1 to indicate that it wasn't 
     found */
  FIND FIRST ttRSSfeed WHERE ttRSSfeed.FeedID = pFeedId NO-ERROR.  
  IF NOT AVAIL ttRSSfeed THEN RETURN -2.

  /* If the Chanel ID wasn't found return -4 to indicate that it wasn't 
     found */
  FIND FIRST ttRSSchannel
       WHERE ttRSSchannel.FeedID    = pFeedID 
         AND ttRSSchannel.ChannelID = pChannelID NO-LOCK NO-ERROR.  
  IF NOT AVAIL ttRSSchannel THEN RETURN -4.

  IF LOOKUP(pOptionName,"{&ChannelOptions}") = 0 THEN 
    RETURN -9.

  CREATE ttRSSoption.
  ASSIGN ttRSSoption.FeedID      = pFeedID
         ttRSSoption.ChannelID   = pChannelID
         ttRSSoption.ItemID      = -1
         ttRSSoption.OptionName  = pOptionName
         ttRSSoption.OptionValue = pOptionValue.

  RETURN 0.
END FUNCTION. /*  SetRSSChannelOption */

FUNCTION SetRSSItemOption RETURN INTEGER
   (INPUT pFeedID       AS INTEGER,
    INPUT pChannelID    AS INTEGER,
    INPUT pItemID       AS INTEGER,
    INPUT pOptionName   AS CHARACTER,
    INPUT pOptionValue  AS CHARACTER):

  /* If the FeedID wasn't found then return -1 to indicate that it wasn't 
     found */
  FIND FIRST ttRSSfeed WHERE ttRSSfeed.FeedID = pFeedId NO-ERROR.  
  IF NOT AVAIL ttRSSfeed THEN RETURN -2.

  /* If the Channel ID wasn't found return -4 to indicate that it wasn't 
     found */
  FIND FIRST ttRSSchannel
       WHERE ttRSSchannel.FeedID    = pFeedID 
         AND ttRSSchannel.ChannelID = pChannelID NO-LOCK NO-ERROR.  
  IF NOT AVAIL ttRSSchannel THEN RETURN -4.

  /* If the Item ID wasn't found return -4 to indicate that it wasn't 
     found */
  FIND FIRST ttRSSitem
       WHERE ttRSSitem.FeedID    = pFeedID 
         AND ttRSSitem.ChannelID = pChannelID 
         AND ttRSSitem.ItemID    = pItemID NO-LOCK NO-ERROR.  
  IF NOT AVAIL ttRSSchannel THEN RETURN -10.

  IF LOOKUP(pOptionName,"{&ItemOptions}") = 0 THEN 
    RETURN -11.

  CREATE ttRSSoption.
  ASSIGN ttRSSoption.FeedID      = pFeedID
         ttRSSoption.ChannelID   = pChannelID
         ttRSSoption.ItemID      = pItemID
         ttRSSoption.OptionName  = pOptionName
         ttRSSoption.OptionValue = pOptionValue.

  RETURN 0.
END FUNCTION. /*  SetRSSItemOption */

FUNCTION CreateRSSItem RETURN INTEGER
   (INPUT pFeedID       AS INTEGER,
    INPUT pChannelID    AS INTEGER,
    INPUT pItemTitle    AS CHARACTER,
    INPUT pItemDesc     AS CHARACTER):

  DEFINE VARIABLE iItem AS INTEGER NO-UNDO.

  DEFINE BUFFER BttRSSitem FOR ttRSSitem.

  /* If the FeedID wasn't found then return -1 to indicate that it wasn't 
     found */
  FIND FIRST ttRSSfeed WHERE ttRSSfeed.FeedID = pFeedId NO-ERROR.  
  IF NOT AVAIL ttRSSfeed THEN RETURN -2.

  /* If the Chanel ID wasn't found return -4 to indicate that it wasn't 
     found */
  FIND FIRST ttRSSchannel
       WHERE ttRSSchannel.FeedID    = pFeedID 
         AND ttRSSchannel.ChannelID = pChannelID NO-LOCK NO-ERROR.  
  IF NOT AVAIL ttRSSchannel THEN RETURN -4.

  IF (pItemTitle = "" AND pItemDesc = "") 
  OR (pItemTitle = ? AND pItemDesc = ?) 
  THEN RETURN -5.

  FIND LAST BttRSSitem NO-LOCK NO-ERROR.
  iItem = IF NOT AVAIL BttRSSitem THEN 1 ELSE BttRSSitem.ItemId + 1.

  CREATE ttRSSitem.
  ASSIGN ttRSSitem.FeedID      = pFeedID
         ttRSSitem.ChannelID   = pChannelID
         ttRSSitem.ItemId      = iItem
         ttRSSitem.ItemTitle   = pItemTitle
         ttRSSitem.ItemDesc    = pItemDesc.

  RETURN iItem.
END FUNCTION. /* CreateItem */

FUNCTION GetRSSError RETURN CHARACTER
   (INPUT pError    AS INTEGER):

  CASE pError:
    WHEN -1 THEN
      RETURN "Feed File Name has already been used!".
    WHEN -2 THEN
      RETURN "Feed ID was not found!".
    WHEN -3 THEN
      RETURN "Channel Name has already been used!".
    WHEN -4 THEN
      RETURN "Channel ID was not found!".
    WHEN -5 THEN
      RETURN "The Channel Title cannot be blank!".
    WHEN -6 THEN
      RETURN "The Channel Link cannot be blank!".
    WHEN -7 THEN
      RETURN "The Channel Description cannot be blank!".
    WHEN -8 THEN
      RETURN "Either the Item Title or the Item Description must be defined!".
    WHEN -9 THEN
      RETURN "Invalid Channel Option specified!".
    WHEN -10 THEN
      RETURN "The Item ID was not found!".
    WHEN -11 THEN
      RETURN "Invalid Item Option specified!".

    OTHERWISE
      RETURN "".
  END CASE.

END FUNCTION. /* GetRSSError */

/* end of rssFeed.i */
